# genetic_variation type associated with detected resistance
NUCLEOTIDE_VARIANT = "nucleotide_variant_detected"
AMINO_ACID_VARIANT = "protein_variant_detected"
GENE_PRESENCE = "gene_presence_detected"
